
local module = D:module('diamond_heist_detection_meter')

local NOT_SUS = 100000

module:hook(GroupAIStateBase, "init", function(self)
	module:call_orig(GroupAIStateBase, "init", self)
	--debug_draw_state is kinda poggers
	--self:set_debug_draw_state(true)
	self._dhdm_guards = {}
	self._dhdm_prevsus = NOT_SUS
	self._dhdm_created_waypoints = {}
end)


module:hook(GroupAIStateBase, "update", function(self, t, dt)
	module:call_orig(GroupAIStateBase, "update", self, t, dt)
	
	--FWB has different rules than DH
	local is_dh = Global.level_data.level_id == "diamond_heist"
	local is_fwb = Global.level_data.level_id == "bank"
	
	--only run this in whisper mode, and only on FWB/DH
	--wipe any potential straggler waypoints when whisper mode is over
	if not is_dh and not is_fwb then return end
	if not managers.groupai:state():whisper_mode() and game_state_machine:current_state_name() ~= "ingame_mask_off" then
		for i, v in pairs(self._dhdm_created_waypoints) do
			managers.hud:remove_waypoint(i)
		end
		self._dhdm_created_waypoints = {}
		return
	end
	
	local cops = managers.enemy:all_enemies()
	
	--Diamond Heist detection goes from 0 (detected) to 1 (detection just started)
	--FWB guard detection is simply a timestamp 2 seconds after the suspicion began
	local highestdetection = NOT_SUS
	
	for _, cop in pairs(cops) do
		local key = tostring(cop.unit:key())
		
		--among us
		local sus = self._dhdm_guards[key]
		if (sus ~= nil) then
			--make sus between 2 and 0 when not diamond heist
			if (not is_dh) then sus = sus - t end
			if (highestdetection > sus) then highestdetection = sus end
			
			--place exclamation point above head; code from Payday 2
			local icon_pos = mvector3.copy(math.UP)
			mvector3.multiply(icon_pos, 28)
			local observer_pos = cop.unit:movement() and cop.unit:movement():m_head_pos()
			if not observer_pos then
				u_observer:m_position(tmp_vec1)
				observer_pos = tmp_vec1
			end
			mvector3.add(icon_pos, observer_pos)
			--
			
			if (managers.hud:get_waypoint_data(key) == nil) then
				managers.hud:add_waypoint(key, {
					text = "!",
					position = icon_pos,
					distance = false,
					present_timer = 0,
				})
				managers.hud._sound_source:post_event("prompt_exit")
				self._dhdm_created_waypoints[key] = true
				--managers.hud:get_waypoint_data(key).move_speed = 0.01
			else
				managers.hud:get_waypoint_data(key).position = icon_pos
			end
		else
			managers.hud:remove_waypoint(key)
		end
	end
	
	
	if (is_dh) then
		--it will never show 100% because you move from the suspect list to the detected list. sudden jump from 90% to 0% = 100%
		if (self._dhdm_prevsus < 0.5 and highestdetection == NOT_SUS) then
			managers.hint:show_hint("dhdm_hint_detection",nil, false, {PERCENTAGE = 100})
			managers.hud._sound_source:post_event("stinger_feedback_negative")
		--it would never show 0% because you're removed from the list entirely
		elseif (highestdetection == NOT_SUS and self._dhdm_prevsus > 0.5 and self._dhdm_prevsus ~= NOT_SUS) then
			managers.hint:show_hint("dhdm_hint_detection",nil, false, {PERCENTAGE = 0})
		elseif (highestdetection ~= NOT_SUS) then
			local percentage = math.min(math.round((1 - highestdetection) * 100), 100)
			managers.hint:show_hint("dhdm_hint_detection",nil, false, {PERCENTAGE = percentage})
		end
	else
		--FWB guard detection instantly goes back to 0%
		if (highestdetection == NOT_SUS and self._dhdm_prevsus ~= NOT_SUS) then
			managers.hint:show_hint("dhdm_hint_detection",nil, false, {PERCENTAGE = 0})
		elseif (highestdetection ~= NOT_SUS) then
			local percentage = math.min(math.round((2 - (highestdetection)) * 100),100)
			managers.hint:show_hint("dhdm_hint_detection",nil, false, {PERCENTAGE = percentage})
		end
	end
	
	self._dhdm_prevsus = highestdetection
end)